@@ -2,7 +2,7 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 import os
 from pathlib import Path
-from pmb.types import AportGenEntry, PathString
+from pmb.types import PathString
 import sys
 from collections.abc import Sequence
 
@@ -14,7 +14,10 @@ from collections.abc import Sequence
 from pmb.config.file import load as load, save as save, serialize as serialize
 from pmb.config.sudo import which_sudo
 from pmb.config.other import is_systemd_selected as is_systemd_selected
-from .init import require_programs as require_programs
+from .init import (
+    require_programs as require_programs,
+    ask_for_mainline_downstream as ask_for_mainline_downstream,
+)
 from . import workdir as workdir
 
 
@@ -30,7 +33,8 @@ apk_keys_path: Path = pmb_src / "pmb/data/keys"
 # (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
 apk_tools_min_version = {
-    "edge": "2.14.9-r1",
+    "edge": "3.0.0_rc5_git20250819",
+    "v3.22": "2.14.9-r3",
     "v3.21": "2.14.6-r3",
     "v3.20": "2.14.4-r1",
     "v3.19": "2.14.4-r0",
@@ -104,13 +108,14 @@ styles = {
     "BLUE": "\033[94m",
     "BOLD": "\033[1m",
     "GREEN": "\033[92m",
+    "MAGENTA": "\033[95m",
     "RED": "\033[91m",
     "YELLOW": "\033[93m",
     "END": "\033[0m",
 }
 
 if "NO_COLOR" in os.environ:
-    for style in styles.keys():
+    for style in styles:
         styles[style] = ""
 
 # Supported filesystems and their fstools packages
@@ -218,11 +223,7 @@ chroot_outdated = 3600 * 24 * 2
 # for the first time
 # IMPORTANT: the order here matters, it is the order these packages will
 # be built in (if needed). abuild must be first!
-#
-# NOTE: full hexdump is installed to workaround a bug in busybox,
-# see https://gitlab.postmarketos.org/postmarketOS/pmaports/-/issues/3268. This can be
-# reverted when the bug is properly fixed.
-build_packages = ["abuild", "apk-tools", "build-base", "ccache", "git", "hexdump", "pigz"]
+build_packages = ["abuild", "apk-tools", "build-base", "ccache", "git", "pigz"]
 
 #
 # PARSE
@@ -284,6 +285,7 @@ apkbuild_attributes = {
     "_llvmver": {},
     # Overridden packages
     "_pkgver": {},
+    "_pkgrel": {},
     "_pkgname": {},
     # git commit
     "_commit": {},
@@ -373,6 +375,7 @@ $IMAGE: Path to the combined boot/rootfs image
 $IMAGE_SPLIT_BOOT: Path to the (split) boot image
 $IMAGE_SPLIT_ROOT: Path to the (split) rootfs image
 $PARTITION_KERNEL: Partition to flash the kernel/boot.img to
+$PARTITION_VENDOR_BOOT: Partition to flash the vendor_boot.img to
 $PARTITION_ROOTFS: Partition to flash the rootfs to
 
 Fastboot specific: $KERNEL_CMDLINE
@@ -386,6 +389,9 @@ flashers: dict[str, dict[str, bool | list[str] | dict[str, list[list[str]]]]] =
             "list_devices": [["fastboot", "devices", "-l"]],
             "flash_rootfs": [["fastboot", "flash", "$PARTITION_ROOTFS", "$IMAGE"]],
             "flash_kernel": [["fastboot", "flash", "$PARTITION_KERNEL", "$BOOT/boot.img$FLAVOR"]],
+            "flash_vendorboot": [
+                ["fastboot", "flash", "$PARTITION_VENDOR_BOOT", "$BOOT/vendor_boot.img"]
+            ],
             "flash_vbmeta": [
                 # Generate vbmeta image with "disable verification" flag
                 [
@@ -403,6 +409,11 @@ flashers: dict[str, dict[str, bool | list[str] | dict[str, list[list[str]]]]] =
             ],
             "flash_dtbo": [["fastboot", "flash", "$PARTITION_DTBO", "$BOOT/dtbo.img"]],
             "boot": [["fastboot", "--cmdline", "$KERNEL_CMDLINE", "boot", "$BOOT/boot.img$FLAVOR"]],
+            # Like boot, but stage vendor_boot first
+            "boot_gki": [
+                ["fastboot", "stage", "$BOOT/vendor_boot.img"],
+                ["fastboot", "boot", "$BOOT/boot.img$FLAVOR"],
+            ],
             "flash_lk2nd": [["fastboot", "flash", "$PARTITION_KERNEL", "$BOOT/lk2nd.img"]],
         },
     },
@@ -587,16 +598,6 @@ git_repos = {
 #
 # APORTGEN
 #
-aportgen: dict[str, AportGenEntry] = {
-    "cross": {
-        "prefixes": ["busybox-static", "gcc", "musl", "grub-efi"],
-        "confirm_overwrite": False,
-    },
-    "device/testing": {
-        "prefixes": ["device", "linux"],
-        "confirm_overwrite": True,
-    },
-}
 
 # Use a deterministic mirror URL instead of CDN for aportgen. Otherwise we may
 # generate a pmaport that wraps an apk from Alpine (e.g. musl-armv7) locally
@@ -651,4 +652,4 @@ upgrade_ignore = [
 # CI
 #
 # Valid options for 'pmbootstrap ci', see https://postmarketos.org/pmb-ci
-ci_valid_options = ["native", "slow"]
+ci_valid_options = ["native", "slow", "with-dot-git"]
