@@ -3,11 +3,10 @@
 import os
 import shutil
 from functools import lru_cache
-from typing import Optional
 
 
-@lru_cache()
-def which_sudo() -> Optional[str]:
+@lru_cache
+def which_sudo() -> str | None:
     """Return a command required to run commands as root, if any.
 
     Find whether sudo or doas is installed for commands that require root.
@@ -16,20 +15,24 @@ def which_sudo() -> Optional[str]:
     if os.getuid() == 0:
         return None
 
-    supported_sudos = ['doas', 'sudo']
+    supported_sudos = ["doas", "sudo"]
 
     user_set_sudo = os.getenv("PMB_SUDO")
     if user_set_sudo is not None:
         if shutil.which(user_set_sudo) is None:
-            raise RuntimeError("PMB_SUDO environmental variable is set to"
-                               f" {user_set_sudo} but pmbootstrap cannot find"
-                               " this command on your system.")
+            raise RuntimeError(
+                "PMB_SUDO environmental variable is set to"
+                f" {user_set_sudo} but pmbootstrap cannot find"
+                " this command on your system."
+            )
         return user_set_sudo
 
     for sudo in supported_sudos:
         if shutil.which(sudo) is not None:
             return sudo
 
-    raise RuntimeError("Can't find sudo or doas required to run pmbootstrap."
-                       " Please install sudo, doas, or specify your own sudo"
-                       " with the PMB_SUDO environmental variable.")
+    raise RuntimeError(
+        "Can't find sudo or doas required to run pmbootstrap."
+        " Please install sudo, doas, or specify your own sudo"
+        " with the PMB_SUDO environmental variable."
+    )
