@@ -19,7 +19,7 @@ import pmb.chroot.other
 import pmb.ci
 import pmb.config
 from pmb.core import Config
-from pmb.types import Env, PmbArgs
+from pmb.types import Env, PmbArgs, RunOutputTypeDefault
 import pmb.export
 import pmb.flasher
 import pmb.helpers.aportupgrade
@@ -77,8 +77,8 @@ def _parse_suffix(args: PmbArgs) -> Chroot:
         else:
             return Chroot.buildroot(Arch.from_str(args.buildroot))
     elif args.suffix:
-        (_t, s) = args.suffix.split("_")
-        t: ChrootType = ChrootType(_t)
+        (t_, s) = args.suffix.split("_")
+        t: ChrootType = ChrootType(t_)
         return Chroot(t, s)
     else:
         return Chroot(ChrootType.NATIVE)
@@ -119,11 +119,6 @@ def build(args: PmbArgs) -> None:
     if src and not os.path.exists(src):
         raise RuntimeError("Invalid path specified for --src: " + src)
 
-    # Ensure repo_bootstrap is done for all arches we intend to build for
-    for package in args.packages:
-        arch_package = args.arch or pmb.build.autodetect.arch(package)
-        pmb.helpers.pmaports.require_bootstrap(arch_package, f"build {package} for {arch_package}")
-
     context = get_context()
     # Build all packages
     built = pmb.build.packages(
@@ -262,10 +257,7 @@ def config(args: PmbArgs) -> None:
             print(f"{args.name} = {args.value}")
         pmb.config.save(args.config, config)
     elif args.name:
-        if hasattr(config, args.name):
-            value = getattr(config, args.name)
-        else:
-            value = ""
+        value = getattr(config, args.name) if hasattr(config, args.name) else ""
 
         def to_shell_friendly_representation(value: Any) -> str:
             friendly_representation: str
@@ -273,10 +265,7 @@ def config(args: PmbArgs) -> None:
             if isinstance(value, list) and len(value) == 1:
                 value = value[0]
 
-            if isinstance(value, Path):
-                friendly_representation = value.as_posix()
-            else:
-                friendly_representation = str(value)
+            friendly_representation = value.as_posix() if isinstance(value, Path) else str(value)
 
             return friendly_representation
 
@@ -321,10 +310,6 @@ def install(args: PmbArgs) -> None:
     if args.rsync and args.filesystem == "btrfs":
         raise ValueError("Installation using rsync is not currently supported on btrfs filesystem.")
 
-    pmb.helpers.pmaports.require_bootstrap(
-        deviceinfo.arch, f"do 'pmbootstrap install' for {deviceinfo.arch} (deviceinfo_arch)"
-    )
-
     # On-device installer checks
     # Note that this can't be in the mutually exclusive group that has most of
     # the conflicting options, because then it would not work with --disk.
@@ -360,19 +345,18 @@ def install(args: PmbArgs) -> None:
         _install_ondev_verify_no_rootfs(device, args.ondev_cp)
 
     # On-device installer overrides
-    if args.on_device_installer:
+    if args.on_device_installer and config.user != "user":
         # To make code for the on-device installer not needlessly complex, just
         # hardcode "user" as username here. (The on-device installer will set
         # a password for the user, disable SSH password authentication,
         # optionally add a new user for SSH that must not have the same
         # username etc.)
-        if config.user != "user":
-            logging.warning(
-                f"WARNING: custom username '{config.user}' will be"
-                " replaced with 'user' for the on-device"
-                " installer."
-            )
-            config.user = "user"
+        logging.warning(
+            f"WARNING: custom username '{config.user}' will be"
+            " replaced with 'user' for the on-device"
+            " installer."
+        )
+        config.user = "user"
 
     if not args.disk and args.split is None:
         # Default to split if the flash method requires it
@@ -522,10 +506,12 @@ def stats(args: PmbArgs) -> None:
     # Chroot suffix
     chroot = Chroot.buildroot(args.arch or Arch.native())
 
+    pmb.chroot.init(chroot)
+
     # Install ccache and display stats
     pmb.chroot.apk.install(["ccache"], chroot)
     logging.info(f"({chroot}) % ccache -s")
-    pmb.chroot.user(["ccache", "-s"], chroot, output="stdout")
+    pmb.chroot.user(["ccache", "-s"], chroot, output=RunOutputTypeDefault.STDOUT)
 
 
 def work_migrate(args: PmbArgs) -> None:
@@ -558,6 +544,10 @@ def bootimg_analyze(args: PmbArgs) -> None:
 
 
 def lint(args: PmbArgs) -> None:
+    logging.warning(
+        "WARNING: The 'pmbootstrap lint' command is deprecated. If you are linting pmaports, use 'pmbootstrap ci apkbuild-lint' instead."
+    )
+
     packages: Sequence[str] = args.packages
     if not packages:
         packages = pmb.helpers.pmaports.get_list()
