@@ -49,7 +49,7 @@ def parse_next_block(path: Path, lines: list[str]) -> ApkindexBlock | None:
             continue
         # Parse keys from the mapping
         k = line[0]
-        key = apkindex_map.get(k, None)
+        key = apkindex_map.get(k)
 
         # The checksum key is always the FIRST in the block, so when we find
         # it we know we're done.
@@ -247,22 +247,24 @@ def parse(
 
     # Try to get a cached result first
     lastmod = path.lstat().st_mtime
-    _cache_key = "multiple" if multiple_providers else "single"
+    cache_key_ = "multiple" if multiple_providers else "single"
     key = cache_key(path)
     if key in pmb.helpers.other.cache["apkindex"]:
         cache = pmb.helpers.other.cache["apkindex"][key]
         if cache["lastmod"] == lastmod:
-            if _cache_key in cache:
-                return cache[_cache_key]
+            if cache_key_ in cache:
+                return cache[cache_key_]
         else:
             clear_cache(path)
 
     # Read all lines
     lines: Sequence[str]
     if tarfile.is_tarfile(path):
-        with tarfile.open(path, "r:gz") as tar:
-            with tar.extractfile(tar.getmember("APKINDEX")) as handle:  # type:ignore[union-attr]
-                lines = handle.read().decode().splitlines()
+        with (
+            tarfile.open(path, "r:gz") as tar,  # type:ignore[union-attr]
+            tar.extractfile(tar.getmember("APKINDEX")) as handle,  # type:ignore[union-attr]
+        ):
+            lines = handle.read().decode().splitlines()
     else:
         with path.open("r", encoding="utf-8") as handle:
             lines = handle.read().splitlines()
@@ -296,7 +298,7 @@ def parse(
     key = cache_key(path)
     if key not in pmb.helpers.other.cache["apkindex"]:
         pmb.helpers.other.cache["apkindex"][key] = {"lastmod": lastmod}
-    pmb.helpers.other.cache["apkindex"][key][_cache_key] = ret
+    pmb.helpers.other.cache["apkindex"][key][cache_key_] = ret
     return ret
 
 
@@ -310,9 +312,8 @@ def parse_blocks(path: Path) -> list[ApkindexBlock]:
               parse() if you need these features).
     """
     # Parse all lines
-    with tarfile.open(path, "r:gz") as tar:
-        with tar.extractfile(tar.getmember("APKINDEX")) as handle:  # type:ignore[union-attr]
-            lines = handle.read().decode().splitlines()
+    with tarfile.open(path, "r:gz") as tar, tar.extractfile(tar.getmember("APKINDEX")) as handle:  # type:ignore[union-attr]
+        lines = handle.read().decode().splitlines()
 
     # Parse lines into blocks
     ret: list[ApkindexBlock] = []
@@ -483,6 +484,16 @@ def package(
     if package in package_providers:
         return package_providers[package]
 
+    if package_providers:
+        providers_priority = provider_highest_priority(package_providers, package)
+        num_providers = len(providers_priority)
+        # Only one provider with max priority: return it
+        if num_providers == 1:
+            return next(iter(providers_priority.values()))
+        # Multiple providers with max priority: let follow-on logic decide
+        elif num_providers > 1:
+            package_providers = providers_priority
+
     # Any provider
     if package_providers:
         return pmb.parse.apkindex.provider_shortest(package_providers, package)
