@@ -2,6 +2,7 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 import argparse
 from collections.abc import Sequence
+import contextlib
 import os
 from pathlib import Path
 import sys
@@ -9,13 +10,11 @@ from typing import Any, cast
 
 from pmb.core.arch import Arch
 from pmb.core import Config
-from pmb.types import PmbArgs
+from pmb.types import PmbArgs, RunOutputTypeDefault
 from pmb.helpers.exceptions import NonBugError
 
-try:
+with contextlib.suppress(ImportError):
     import argcomplete  # type:ignore[import-untyped]
-except ImportError:
-    pass
 
 import pmb.config
 import pmb.helpers.args
@@ -476,18 +475,16 @@ def arguments_initfs(subparser: argparse._SubParsersAction) -> argparse.Argument
 def arguments_qemu(subparser: argparse._SubParsersAction) -> argparse.ArgumentParser:
     ret = subparser.add_parser("qemu")
     ret.add_argument("--cmdline", help="override kernel commandline")
-    ret.add_argument(
-        "--image-size", default="4G", help="set rootfs size, e.g. 2048M or 2G (default: 4G)"
-    )
+    ret.add_argument("--image-size", help="set rootfs size (e.g. 2048M or 2G)")
     ret.add_argument(
         "--second-storage",
         metavar="IMAGE_SIZE",
         help="add a second storage with the given size (default:"
-        " 4G), gets created if it does not exist. Use to"
+        " 8G), gets created if it does not exist. Use to"
         " test install from SD to eMMC",
         nargs="?",
         default=None,
-        const="4G",
+        const="8G",
     )
     ret.add_argument("-m", "--memory", type=int, default=1024, help="guest RAM (default: 1024)")
     ret.add_argument("-p", "--port", type=int, default=2222, help="SSH port (default: 2222)")
@@ -729,15 +726,6 @@ def arguments_kconfig(subparser: argparse._SubParsersAction) -> None:
     add_kernel_arg(migrate, nargs=1)
 
 
-def arguments_repo_bootstrap(subparser: argparse._SubParsersAction) -> argparse.ArgumentParser:
-    arch_choices = Arch.supported()
-
-    ret = subparser.add_parser("repo_bootstrap")
-    ret.add_argument("repository", help="which repository to bootstrap (e.g. systemd)")
-    ret.add_argument("--arch", choices=arch_choices, dest="arch", type=lambda x: Arch.from_str(x))
-    return ret
-
-
 def arguments_repo_missing(subparser: argparse._SubParsersAction) -> argparse.ArgumentParser:
     ret = subparser.add_parser(
         "repo_missing",
@@ -824,7 +812,7 @@ def kernel_completer(
     ret: list[str] = []
 
     # Full package name, starting with "linux-"
-    if len("linux-") < len(prefix) and prefix.startswith("linux-") or "linux-".startswith(prefix):
+    if (len("linux-") < len(prefix) and prefix.startswith("linux-")) or "linux-".startswith(prefix):
         ret += package_completer(prefix, action, parser, parsed_args)
 
     # Kernel name without "linux-"
@@ -972,7 +960,6 @@ def get_parser() -> argparse.ArgumentParser:
 
     # Actions
     sub = parser.add_subparsers(title="action", dest="action")
-    sub.add_parser("init", help="initialize config file")
     sub.add_parser("shutdown", help="umount, unregister binfmt")
     sub.add_parser(
         "index",
@@ -985,7 +972,6 @@ def get_parser() -> argparse.ArgumentParser:
         " non-interactively to migrate the"
         " work folder version on demand",
     )
-    arguments_repo_bootstrap(sub)
     arguments_repo_missing(sub)
     arguments_kconfig(sub)
     arguments_export(sub)
@@ -1003,6 +989,14 @@ def get_parser() -> argparse.ArgumentParser:
     arguments_status(sub)
     arguments_ci(sub)
 
+    # Action: init
+    init = sub.add_parser("init", help="initialize config file")
+    init.add_argument(
+        "--shallow-initial-clone",
+        help="do a shallow clone if pmaports has to be cloned. Primarily useful to speed up scripts.",
+        action="store_true",
+    )
+
     # Action: log
     log = sub.add_parser("log", help="follow the pmbootstrap logfile")
     log.add_argument("-n", "--lines", type=int, default=60, help="count of initial output lines")
@@ -1098,12 +1092,13 @@ def get_parser() -> argparse.ArgumentParser:
     chroot.add_argument("--user", help="run the command as user, not as root", action="store_true")
     chroot.add_argument(
         "--output",
-        choices=["log", "stdout", "interactive", "tui", "background"],
+        choices=list(RunOutputTypeDefault),
+        type=RunOutputTypeDefault.from_string,
         help="how the output of the"
         " program should be handled, choose from: 'log',"
         " 'stdout', 'interactive', 'tui' (default),"
         " 'background'. Details: pmb/helpers/run_core.py",
-        default="tui",
+        default=RunOutputTypeDefault.TUI,
     )
     chroot.add_argument(
         "--image",
@@ -1345,7 +1340,7 @@ def arguments() -> PmbArgs:
         )
 
     if getattr(args, "go_mod_cache", None) is None:
-        gomodcache = True if getattr(args, "src", None) else False
+        gomodcache = bool(getattr(args, "src", None))
         setattr(args, "go_mod_cache", gomodcache)
 
     return args
