@@ -1,35 +1,21 @@
 # Copyright 2023 Attila Szollosi
 # SPDX-License-Identifier: GPL-3.0-or-later
-import glob
-import logging
+from pathlib import Path
+from pmb.helpers import logging
 import re
 import os
+from typing import Literal, overload
 
 import pmb.build
 import pmb.config
 import pmb.parse
 import pmb.helpers.pmaports
+import pmb.parse.kconfigcheck
 from pmb.helpers.exceptions import NonBugError
+from pmb.types import PathString
 
 
-def get_all_component_names():
-    """
-    Get the component names from kconfig_options variables in
-    pmb/config/__init__.py. This does not include the base options.
-
-    :returns: a list of component names, e.g. ["waydroid", "iwd", "nftables"]
-    """
-    prefix = "kconfig_options_"
-    ret = []
-
-    for key in pmb.config.__dict__.keys():
-        if key.startswith(prefix):
-            ret += [key.split(prefix, 1)[1]]
-
-    return ret
-
-
-def is_set(config, option):
+def is_set(config: str, option: str) -> bool:
     """
     Check, whether a boolean or tristate option is enabled
     either as builtin or module.
@@ -38,10 +24,10 @@ def is_set(config, option):
     :param option: name of the option to check, e.g. EXT4_FS
     :returns: True if the check passed, False otherwise
     """
-    return re.search("^CONFIG_" + option + "=[ym]$", config, re.M) is not None
+    return re.search("^CONFIG_" + option + "=[ym]$", config, re.MULTILINE) is not None
 
 
-def is_set_str(config, option, string):
+def is_set_str(config: str, option: str, string: str) -> bool:
     """
     Check, whether a config option contains a string as value.
 
@@ -50,14 +36,14 @@ def is_set_str(config, option, string):
     :param string: the expected string
     :returns: True if the check passed, False otherwise
     """
-    match = re.search("^CONFIG_" + option + "=\"(.*)\"$", config, re.M)
+    match = re.search("^CONFIG_" + option + '="(.*)"$', config, re.MULTILINE)
     if match:
         return string == match.group(1)
     else:
         return False
 
 
-def is_in_array(config, option, string):
+def is_in_array(config: str, option: str, string: str) -> bool:
     """
     Check, whether a config option contains string as an array element
 
@@ -66,7 +52,7 @@ def is_in_array(config, option, string):
     :param string: the string expected to be an element of the array
     :returns: True if the check passed, False otherwise
     """
-    match = re.search("^CONFIG_" + option + "=\"(.*)\"$", config, re.M)
+    match = re.search("^CONFIG_" + option + '="(.*)"$', config, re.MULTILINE)
     if match:
         values = match.group(1).split(",")
         return string in values
@@ -74,8 +60,14 @@ def is_in_array(config, option, string):
         return False
 
 
-def check_option(component, details, config, config_path, option,
-                 option_value):
+def check_option(
+    component: str,
+    details: bool,
+    config: str,
+    config_path: PathString,
+    option: str,
+    option_value: bool | str | list[str],
+) -> bool:
     """
     Check, whether one kernel config option has a given value.
 
@@ -87,38 +79,80 @@ def check_option(component, details, config, config_path, option,
     :param option_value: expected value, e.g. True, "str", ["str1", "str2"]
     :returns: True if the check passed, False otherwise
     """
-    def warn_ret_false(should_str):
+
+    def warn_ret_false(should_str: str) -> bool:
         config_name = os.path.basename(config_path)
         if details:
-            logging.warning(f"WARNING: {config_name}: CONFIG_{option} should"
-                            f" {should_str} ({component}):"
-                            f" https://wiki.postmarketos.org/wiki/kconfig#CONFIG_{option}")
+            logging.warning(
+                f"WARNING: {config_name}: CONFIG_{option} should {should_str} ({component})"
+            )
         else:
-            logging.warning(f"WARNING: {config_name} isn't configured properly"
-                            f" ({component}), run 'pmbootstrap kconfig check'"
-                            " for details!")
+            logging.warning(
+                f"WARNING: {config_name} isn't configured properly"
+                f" ({component}), run 'pmbootstrap kconfig check'"
+                " for details!"
+            )
         return False
 
+    def warn_ret_true(should_str: str) -> bool:
+        config_name = os.path.basename(config_path)
+        if details:
+            logging.warning(
+                f"INFO: {config_name}: CONFIG_{option} should {should_str} ({component})"
+            )
+        else:
+            logging.warning(
+                f"INFO: {config_name} has suboptimal configuration"
+                f" ({component}), run 'pmbootstrap kconfig check'"
+                " for details!"
+            )
+        return True
+
     if isinstance(option_value, list):
         for string in option_value:
             if not is_in_array(config, option, string):
                 return warn_ret_false(f'contain "{string}"')
     elif isinstance(option_value, str):
-        if not is_set_str(config, option, option_value):
-            return warn_ret_false(f'be set to "{option_value}"')
+        if option_value in ["y", "m", "n"]:
+            # Tristate option
+            if option_value == "n":
+                if is_set(config, option):
+                    return warn_ret_false("*not* be set")
+            else:
+                if not is_set(config, option):
+                    return warn_ret_false("be set (either built-in or module)")
+                # Check if it matches preference and don't fail if there's
+                # a mismatch
+                elif option_value == "y" and not is_set_str(config, option, "y"):
+                    warn_ret_true("be built-in (y) but is module (m)")
+                elif option_value == "m" and is_set_str(config, option, "y"):
+                    warn_ret_true("be module (m) but is built-in (y)")
+        else:
+            # Regular string option
+            if not is_set_str(config, option, option_value):
+                return warn_ret_false(f'be set to "{option_value}"')
     elif option_value in [True, False]:
         if option_value != is_set(config, option):
             return warn_ret_false("be set" if option_value else "*not* be set")
     else:
-        raise RuntimeError("kconfig check code can only handle booleans,"
-                           f" strings and arrays. Given value {option_value}"
-                           " is not supported. If you need this, please patch"
-                           " pmbootstrap or open an issue.")
+        raise RuntimeError(
+            "kconfig check code can only handle booleans,"
+            f" strings and arrays. Given value {option_value}"
+            " is not supported. If you need this, please patch"
+            " pmbootstrap or open an issue."
+        )
     return True
 
 
-def check_config_options_set(config, config_path, config_arch, options,
-                             component, pkgver, details=False):
+def check_config_options_set(
+    config: str,
+    config_path: PathString,
+    config_arch: str,  # TODO: Replace with Arch type?
+    options: dict[str, dict],
+    component: str,
+    pkgver: str,
+    details: bool = False,
+) -> bool:
     """
     Check, whether all the kernel config passes all rules of one component.
 
@@ -127,13 +161,7 @@ def check_config_options_set(config, config_path, config_arch, options,
     :param config: full kernel config as string
     :param config_path: full path to kernel config file
     :param config_arch: architecture name (alpine format, e.g. aarch64, x86_64)
-    :param options: kconfig_options* var passed from pmb/config/__init__.py:
-                    kconfig_options_example = {
-                    ">=0.0.0": {  # all versions
-                    "all": {  # all arches
-                    "ANDROID_PARANOID_NETWORK": False,
-                    },
-                    }
+    :param options: dictionary returned by pmb.parse.kconfigcheck.read_category().
     :param component: name of the component to test (postmarketOS, waydroid, …)
     :param pkgver: kernel version
     :param details: print all warnings if True, otherwise one per component
@@ -151,7 +179,7 @@ def check_config_options_set(config, config_path, config_arch, options,
         if skip:
             continue
 
-        for archs, options in archs_options.items():
+        for archs, arch_options in archs_options.items():
             if archs != "all":
                 # Split and check if the device's architecture architecture has
                 # special config options. If option does not contain the
@@ -160,9 +188,8 @@ def check_config_options_set(config, config_path, config_arch, options,
                 if config_arch not in architectures:
                     continue
 
-            for option, option_value in options.items():
-                if not check_option(component, details, config, config_path,
-                                    option, option_value):
+            for option, option_value in arch_options.items():
+                if not check_option(component, details, config, config_path, option, option_value):
                     ret = False
                     # Stop after one non-detailed error
                     if not details:
@@ -170,66 +197,69 @@ def check_config_options_set(config, config_path, config_arch, options,
     return ret
 
 
-def check_config(config_path, config_arch, pkgver, components_list=[],
-                 details=False, enforce_check=True):
+# TODO: This should probably use Arch and not str for config_arch
+def check_config(
+    config_path: PathString,
+    config_arch: str,
+    pkgver: str,
+    categories: list[str],
+    details: bool = False,
+) -> bool:
     """
     Check, whether one kernel config passes the rules of multiple components.
 
     :param config_path: full path to kernel config file
     :param config_arch: architecture name (alpine format, e.g. aarch64, x86_64)
     :param pkgver: kernel version
-    :param components_list: what to check for, e.g. ["waydroid", "iwd"]
+    :param categories: what to check for, e.g. ["waydroid", "iwd"]
     :param details: print all warnings if True, otherwise one per component
-    :param enforce_check: set to False to not fail kconfig check as long as
-                          everything in kconfig_options is set correctly, even
-                          if additional components are checked
     :returns: True if the check passed, False otherwise
     """
     logging.debug(f"Check kconfig: {config_path}")
     with open(config_path) as handle:
         config = handle.read()
 
-    # Devices in all categories need basic options
-    # https://wiki.postmarketos.org/wiki/Device_categorization
-    components_list = ["postmarketOS"] + components_list
+    if "default" not in categories:
+        categories += ["default"]
 
-    # Devices in "community" or "main" need additional options
-    if "community" in components_list:
-        components_list += [
-            "containers",
-            "filesystems",
-            "iwd",
-            "netboot",
-            "nftables",
-            "usb_gadgets",
-            "waydroid",
-            "wireguard",
-            "zram",
+    # Get all rules
+    rules: dict = {}
+    for category in categories:
+        rules |= pmb.parse.kconfigcheck.read_category(category)
+
+    # Check the rules of each category
+    ret = []
+    for category in rules:
+        ret += [
+            check_config_options_set(
+                config, config_path, config_arch, rules[category], category, pkgver, details
+            )
         ]
 
-    components = {}
-    for name in components_list:
-        if name == "postmarketOS":
-            pmb_config_var = "kconfig_options"
-        else:
-            pmb_config_var = f"kconfig_options_{name}"
-
-        components[name] = getattr(pmb.config, pmb_config_var, None)
-        assert components[name], f"invalid kconfig component name: {name}"
-
-    results = []
-    for component, options in components.items():
-        result = check_config_options_set(config, config_path, config_arch,
-                                          options, component, pkgver, details)
-        # We always enforce "postmarketOS" component and when explicitly
-        # requested
-        if enforce_check or component == "postmarketOS":
-            results += [result]
-
-    return all(results)
+    return all(ret)
 
 
-def check(args, pkgname, components_list=[], details=False, must_exist=True):
+@overload
+def check(
+    pkgname: str,
+    components_list: list[str] = ...,
+    details: bool = ...,
+    must_exist: Literal[False] = ...,
+) -> bool | None: ...
+
+
+@overload
+def check(
+    pkgname: str,
+    components_list: list[str] = ...,
+    details: bool = ...,
+    must_exist: Literal[True] = ...,
+) -> bool: ...
+
+
+def check(
+    pkgname: str, components_list: list[str] = [], details: bool = False, must_exist: bool = True
+) -> bool | None:
     """
     Check for necessary kernel config options in a package.
 
@@ -245,48 +275,57 @@ def check(args, pkgname, components_list=[], details=False, must_exist=True):
     components_list = components_list.copy()
 
     # Pkgname: allow omitting "linux-" prefix
-    if pkgname.startswith("linux-"):
-        flavor = pkgname.split("linux-")[1]
-    else:
-        flavor = pkgname
+    flavor = pkgname.split("linux-")[1] if pkgname.startswith("linux-") else pkgname
 
     # Read all kernel configs in the aport
     ret = True
-    aport = pmb.helpers.pmaports.find(args, "linux-" + flavor, must_exist=must_exist)
-    if aport is None:
+    aport: Path
+    try:
+        aport = pmb.helpers.pmaports.find("linux-" + flavor)
+    except RuntimeError as e:
+        if must_exist:
+            raise e
         return None
-    apkbuild = pmb.parse.apkbuild(f"{aport}/APKBUILD")
+    apkbuild = pmb.parse.apkbuild(aport / "APKBUILD")
     pkgver = apkbuild["pkgver"]
 
-    # We only enforce optional checks for community & main devices
-    enforce_check = aport.split("/")[-2] in ["community", "main"]
+    # Get categories from the APKBUILD
+    categories = []
+    for option in apkbuild["options"]:
+        if not option.startswith("pmb:kconfigcheck-"):
+            continue
+        category = option.split("-", 1)[1]
+        categories += [category]
 
-    for name in get_all_component_names():
-        if f"pmb:kconfigcheck-{name}" in apkbuild["options"] and \
-                name not in components_list:
-            components_list += [name]
-
-    for config_path in glob.glob(aport + "/config-*"):
+    for config_path in aport.glob("config-*"):
         # The architecture of the config is in the name, so it just needs to be
         # extracted
         config_name = os.path.basename(config_path)
         config_name_split = config_name.split(".")
 
         if len(config_name_split) != 2:
-            raise NonBugError(f"{config_name} is not a valid kernel config"
-                              "name. Ensure that the _config property in your "
-                              "kernel APKBUILD has a . before the "
-                              "architecture name, e.g. .aarch64 or .armv7, "
-                              "and that there is no excess punctuation "
-                              "elsewhere in the name.")
+            raise NonBugError(
+                f"{config_name} is not a valid kernel config"
+                "name. Ensure that the _config property in your "
+                "kernel APKBUILD has a . before the "
+                "architecture name, e.g. .aarch64 or .armv7, "
+                "and that there is no excess punctuation "
+                "elsewhere in the name."
+            )
 
         config_arch = config_name_split[1]
-        ret &= check_config(config_path, config_arch, pkgver, components_list,
-                            details=details, enforce_check=enforce_check)
+        ret &= check_config(
+            config_path,
+            config_arch,
+            pkgver,
+            categories,
+            details=details,
+        )
     return ret
 
 
-def extract_arch(config_path):
+# TODO: Make this use the Arch type probably
+def extract_arch(config_path: PathString) -> str:
     # Extract the architecture out of the config
     with open(config_path) as f:
         config = f.read()
@@ -306,7 +345,7 @@ def extract_arch(config_path):
     return "unknown"
 
 
-def extract_version(config_path):
+def extract_version(config_path: PathString) -> str:
     # Try to extract the version string out of the comment header
     with open(config_path) as f:
         # Read the first 3 lines of the file and get the third line only
@@ -320,7 +359,9 @@ def extract_version(config_path):
     return "unknown"
 
 
-def check_file(config_path, components_list=[], details=False):
+def check_file(
+    config_path: PathString, components_list: list[str] = [], details: bool = False
+) -> bool:
     """
     Check for necessary kernel config options in a kconfig file.
 
@@ -331,7 +372,5 @@ def check_file(config_path, components_list=[], details=False):
     """
     arch = extract_arch(config_path)
     version = extract_version(config_path)
-    logging.debug(f"Check kconfig: parsed arch={arch}, version={version} from "
-                  f"file: {config_path}")
-    return check_config(config_path, arch, version, components_list,
-                        details=details)
+    logging.debug(f"Check kconfig: parsed arch={arch}, version={version} from file: {config_path}")
+    return check_config(config_path, arch, version, components_list, details=details)
